@@ -12,7 +12,6 @@ import (
 	"github.com/brimdata/zed/cli/runtimeflags"
 	"github.com/brimdata/zed/compiler"
 	"github.com/brimdata/zed/pkg/charm"
-	"github.com/brimdata/zed/pkg/rlimit"
 	"github.com/brimdata/zed/pkg/storage"
 	"github.com/brimdata/zed/runtime"
 	"github.com/brimdata/zed/zbuf"
@@ -138,9 +137,6 @@ func (c *Command) Run(args []string) error {
 		fmt.Println(zfmt.AST(flowgraph))
 		return nil
 	}
-	if _, err := rlimit.RaiseOpenFilesLimit(); err != nil {
-		return err
-	}
 	zctx := zed.NewContext()
 	local := storage.NewLocalEngine()
 	var readers []zio.Reader
