@@ -18,16 +18,17 @@ import (
 	_ "github.com/brimdata/zed/cmd/zed/dev/indexfile"
 	_ "github.com/brimdata/zed/cmd/zed/dev/indexfile/create"
 	_ "github.com/brimdata/zed/cmd/zed/dev/indexfile/lookup"
-	_ "github.com/brimdata/zed/cmd/zed/dev/zst"
-	_ "github.com/brimdata/zed/cmd/zed/dev/zst/create"
-	_ "github.com/brimdata/zed/cmd/zed/dev/zst/cut"
-	_ "github.com/brimdata/zed/cmd/zed/dev/zst/read"
+	_ "github.com/brimdata/zed/cmd/zed/dev/vcache/copy"
+	_ "github.com/brimdata/zed/cmd/zed/dev/vcache/project"
 	"github.com/brimdata/zed/cmd/zed/drop"
 	"github.com/brimdata/zed/cmd/zed/index"
 	zedinit "github.com/brimdata/zed/cmd/zed/init"
 	"github.com/brimdata/zed/cmd/zed/load"
 	"github.com/brimdata/zed/cmd/zed/log"
 	"github.com/brimdata/zed/cmd/zed/ls"
+	"github.com/brimdata/zed/cmd/zed/manage"
+	_ "github.com/brimdata/zed/cmd/zed/manage/monitor"
+	_ "github.com/brimdata/zed/cmd/zed/manage/update"
 	"github.com/brimdata/zed/cmd/zed/merge"
 	"github.com/brimdata/zed/cmd/zed/query"
 	"github.com/brimdata/zed/cmd/zed/rename"
@@ -36,6 +37,7 @@ import (
 	"github.com/brimdata/zed/cmd/zed/serve"
 	"github.com/brimdata/zed/cmd/zed/use"
 	"github.com/brimdata/zed/cmd/zed/vacate"
+	"github.com/brimdata/zed/cmd/zed/vector"
 )
 
 func main() {
@@ -51,6 +53,7 @@ func main() {
 	zed.Add(load.Cmd)
 	zed.Add(log.Cmd)
 	zed.Add(ls.Cmd)
+	zed.Add(manage.Cmd)
 	zed.Add(merge.Cmd)
 	zed.Add(query.Cmd)
 	zed.Add(rename.Cmd)
@@ -58,6 +61,7 @@ func main() {
 	zed.Add(serve.Cmd)
 	zed.Add(use.Cmd)
 	zed.Add(vacate.Cmd)
+	zed.Add(vector.Cmd)
 	zed.Add(dev.Cmd)
 	if err := root.Zed.ExecRoot(os.Args[1:]); err != nil {
 		fmt.Fprintf(os.Stderr, "%s\n", err)
