@@ -371,6 +371,7 @@ type (
 		Kind      string   `json:"kind" unpack:""`
 		Spec      PoolSpec `json:"spec"`
 		At        string   `json:"at"`
+		Delete    bool     `json:"delete"`
 		Range     *Range   `json:"range"`
 		ScanOrder string   `json:"scan_order"` // asc, desc, or unknown
 	}
@@ -383,9 +384,9 @@ type (
 )
 
 type PoolSpec struct {
-	Pool   string `json:"pool"`
-	Commit string `json:"commit"`
-	Meta   string `json:"meta"`
+	Pool   Pattern `json:"pool"`
+	Commit string  `json:"commit"`
+	Meta   string  `json:"meta"`
 }
 
 type Range struct {
