@@ -4,6 +4,7 @@ import (
 	"context"
 	"errors"
 	"fmt"
+	"regexp"
 
 	"github.com/brimdata/zed/compiler/ast"
 	"github.com/brimdata/zed/compiler/ast/dag"
@@ -12,6 +13,7 @@ import (
 	"github.com/brimdata/zed/lakeparse"
 	"github.com/brimdata/zed/order"
 	"github.com/brimdata/zed/pkg/field"
+	"github.com/brimdata/zed/pkg/reglob"
 	"github.com/brimdata/zed/runtime/expr/function"
 	"github.com/segmentio/ksuid"
 )
@@ -23,7 +25,7 @@ func semFrom(ctx context.Context, scope *Scope, from *ast.From, source *data.Sou
 		if err != nil {
 			return nil, err
 		}
-		trunks = append(trunks, converted)
+		trunks = append(trunks, converted...)
 	}
 	return &dag.From{
 		Kind:   "From",
@@ -31,47 +33,61 @@ func semFrom(ctx context.Context, scope *Scope, from *ast.From, source *data.Sou
 	}, nil
 }
 
-func semTrunk(ctx context.Context, scope *Scope, trunk ast.Trunk, ds *data.Source, head *lakeparse.Commitish) (dag.Trunk, error) {
-	source, err := semSource(ctx, scope, trunk.Source, ds, head)
+func semTrunk(ctx context.Context, scope *Scope, trunk ast.Trunk, ds *data.Source, head *lakeparse.Commitish) ([]dag.Trunk, error) {
+	if pool, ok := trunk.Source.(*ast.Pool); ok && trunk.Seq != nil {
+		switch pool.Spec.Pool.(type) {
+		case *ast.Glob, *ast.Regexp:
+			return nil, errors.New("=> not allowed after pool pattern in 'from' operator")
+		}
+	}
+	sources, err := semSource(ctx, scope, trunk.Source, ds, head)
 	if err != nil {
-		return dag.Trunk{}, err
+		return nil, err
 	}
 	seq, err := semSequential(ctx, scope, trunk.Seq, ds, head)
 	if err != nil {
-		return dag.Trunk{}, err
+		return nil, err
 	}
-	return dag.Trunk{
-		Kind:   "Trunk",
-		Source: source,
-		Seq:    seq,
-	}, nil
+	var trunks []dag.Trunk
+	for _, source := range sources {
+		trunks = append(trunks, dag.Trunk{
+			Kind:   "Trunk",
+			Source: source,
+			Seq:    seq,
+		})
+	}
+	return trunks, nil
 }
 
 //XXX make sure you can't read files from a lake instance
 
-func semSource(ctx context.Context, scope *Scope, source ast.Source, ds *data.Source, head *lakeparse.Commitish) (dag.Source, error) {
+func semSource(ctx context.Context, scope *Scope, source ast.Source, ds *data.Source, head *lakeparse.Commitish) ([]dag.Source, error) {
 	switch p := source.(type) {
 	case *ast.File:
 		layout, err := semLayout(p.Layout)
 		if err != nil {
 			return nil, err
 		}
-		return &dag.File{
-			Kind:   "File",
-			Path:   p.Path,
-			Format: p.Format,
-			Layout: layout,
+		return []dag.Source{
+			&dag.File{
+				Kind:   "File",
+				Path:   p.Path,
+				Format: p.Format,
+				Layout: layout,
+			},
 		}, nil
 	case *ast.HTTP:
 		layout, err := semLayout(p.Layout)
 		if err != nil {
 			return nil, err
 		}
-		return &dag.HTTP{
-			Kind:   "HTTP",
-			URL:    p.URL,
-			Format: p.Format,
-			Layout: layout,
+		return []dag.Source{
+			&dag.HTTP{
+				Kind:   "HTTP",
+				URL:    p.URL,
+				Format: p.Format,
+				Layout: layout,
+			},
 		}, nil
 	case *ast.Pool:
 		if !ds.IsLake() {
@@ -79,10 +95,10 @@ func semSource(ctx context.Context, scope *Scope, source ast.Source, ds *data.So
 		}
 		return semPool(ctx, scope, p, ds, head)
 	case *ast.Pass:
-		return &dag.Pass{Kind: "Pass"}, nil
+		return []dag.Source{&dag.Pass{Kind: "Pass"}}, nil
 	case *kernel.Reader:
 		// kernel.Reader implements both ast.Source and dag.Source
-		return p, nil
+		return []dag.Source{p}, nil
 	default:
 		return nil, fmt.Errorf("semantic analyzer: unknown AST source type %T", p)
 	}
@@ -107,8 +123,38 @@ func semLayout(p *ast.Layout) (order.Layout, error) {
 	return order.NewLayout(which, keys), nil
 }
 
-func semPool(ctx context.Context, scope *Scope, p *ast.Pool, ds *data.Source, head *lakeparse.Commitish) (dag.Source, error) {
-	poolName := p.Spec.Pool
+func semPool(ctx context.Context, scope *Scope, p *ast.Pool, ds *data.Source, head *lakeparse.Commitish) ([]dag.Source, error) {
+	var poolNames []string
+	var err error
+	switch specPool := p.Spec.Pool.(type) {
+	case nil:
+		// This is a lake meta-query.
+		poolNames = []string{""}
+	case *ast.Glob:
+		poolNames, err = matchPools(ctx, ds, reglob.Reglob(specPool.Pattern), specPool.Pattern, "glob")
+	case *ast.Regexp:
+		poolNames, err = matchPools(ctx, ds, specPool.Pattern, specPool.Pattern, "regexp")
+	case *ast.String:
+		poolNames = []string{specPool.Text}
+	default:
+		return nil, fmt.Errorf("semantic analyzer: unknown AST pool type %T", specPool)
+	}
+	if err != nil {
+		return nil, err
+	}
+	var sources []dag.Source
+	for _, name := range poolNames {
+		source, err := semPoolWithName(ctx, scope, p, name, ds, head)
+		if err != nil {
+			return nil, err
+		}
+		sources = append(sources, source)
+	}
+	return sources, nil
+}
+
+func semPoolWithName(ctx context.Context, scope *Scope, p *ast.Pool, poolName string, ds *data.Source,
+	head *lakeparse.Commitish) (dag.Source, error) {
 	commit := p.Spec.Commit
 	if poolName == "HEAD" {
 		if head == nil {
@@ -152,7 +198,6 @@ func semPool(ctx context.Context, scope *Scope, p *ast.Pool, ds *data.Source, he
 			}
 		}
 	}
-	//var at ksuid.KSUID
 	if p.At != "" {
 		// XXX
 		// We no longer use "at" to refer to a commit tag, but if there
@@ -173,7 +218,7 @@ func semPool(ctx context.Context, scope *Scope, p *ast.Pool, ds *data.Source, he
 		}
 	}
 	if p.Spec.Meta != "" {
-		if commitID != ksuid.Nil {
+		if commit != "" {
 			return &dag.CommitMeta{
 				Kind:      "CommitMeta",
 				Meta:      p.Spec.Meta,
@@ -202,12 +247,34 @@ func semPool(ctx context.Context, scope *Scope, p *ast.Pool, ds *data.Source, he
 		Kind:      "Pool",
 		ID:        poolID,
 		Commit:    commitID,
+		Delete:    p.Delete,
 		ScanLower: lower,
 		ScanUpper: upper,
 		ScanOrder: p.ScanOrder,
 	}, nil
 }
 
+func matchPools(ctx context.Context, ds *data.Source, pattern, origPattern, patternDesc string) ([]string, error) {
+	re, err := regexp.Compile(pattern)
+	if err != nil {
+		return nil, err
+	}
+	pools, err := ds.Lake().ListPools(ctx)
+	if err != nil {
+		return nil, err
+	}
+	var matches []string
+	for _, p := range pools {
+		if re.MatchString(p.Name) {
+			matches = append(matches, p.Name)
+		}
+	}
+	if len(matches) == 0 {
+		return nil, fmt.Errorf("%s: pool matching %s not found", origPattern, patternDesc)
+	}
+	return matches, nil
+}
+
 func semSequential(ctx context.Context, scope *Scope, seq *ast.Sequential, ds *data.Source, head *lakeparse.Commitish) (*dag.Sequential, error) {
 	if seq == nil {
 		return nil, nil
@@ -319,7 +386,7 @@ func semOp(ctx context.Context, scope *Scope, o ast.Op, ds *data.Source, head *l
 			Cases: cases,
 		}, nil
 	case *ast.Shape:
-		return &dag.Shape{"Shape"}, nil
+		return &dag.Shape{Kind: "Shape"}, nil
 	case *ast.Cut:
 		assignments, err := semAssignments(scope, o.Args, false)
 		if err != nil {
@@ -376,7 +443,7 @@ func semOp(ctx context.Context, scope *Scope, o ast.Op, ds *data.Source, head *l
 			Cflag: o.Cflag,
 		}, nil
 	case *ast.Pass:
-		return &dag.Pass{"Pass"}, nil
+		return &dag.Pass{Kind: "Pass"}, nil
 	case *ast.OpExpr:
 		return semOpExpr(scope, o.Expr)
 	case *ast.Search:
@@ -443,14 +510,14 @@ func semOp(ctx context.Context, scope *Scope, o ast.Op, ds *data.Source, head *l
 					return nil, fmt.Errorf("cannot rename %s to %s (differ in %s vs %s)", src, dst, src.Path[i], dst.Path[i])
 				}
 			}
-			assignments = append(assignments, dag.Assignment{"Assignment", dst, src})
+			assignments = append(assignments, dag.Assignment{Kind: "Assignment", LHS: dst, RHS: src})
 		}
 		return &dag.Rename{
 			Kind: "Rename",
 			Args: assignments,
 		}, nil
 	case *ast.Fuse:
-		return &dag.Fuse{"Fuse"}, nil
+		return &dag.Fuse{Kind: "Fuse"}, nil
 	case *ast.Join:
 		leftKey, err := semExpr(scope, o.LeftKey)
 		if err != nil {
@@ -702,7 +769,7 @@ func semCallOp(scope *Scope, call *ast.Call) (dag.Op, error) {
 			Aggs: []dag.Assignment{
 				{
 					Kind: "Assignment",
-					LHS:  &dag.This{"This", field.New(call.Name)},
+					LHS:  &dag.This{Kind: "This", Path: field.New(call.Name)},
 					RHS:  agg,
 				},
 			},
