@@ -34,13 +34,15 @@ type Interface interface {
 	MergeBranch(ctx context.Context, pool ksuid.KSUID, childBranch, parentBranch string, message api.CommitMessage) (ksuid.KSUID, error)
 	Compact(ctx context.Context, pool ksuid.KSUID, branch string, objects []ksuid.KSUID, message api.CommitMessage) (ksuid.KSUID, error)
 	Load(ctx context.Context, zctx *zed.Context, pool ksuid.KSUID, branch string, r zio.Reader, message api.CommitMessage) (ksuid.KSUID, error)
-	Delete(ctx context.Context, pool ksuid.KSUID, branchName string, tags []ksuid.KSUID, message api.CommitMessage) (ksuid.KSUID, error)
-	DeleteByPredicate(ctx context.Context, pool ksuid.KSUID, branchName, src string, commit api.CommitMessage) (ksuid.KSUID, error)
+	Delete(ctx context.Context, poolID ksuid.KSUID, branchName string, tags []ksuid.KSUID, message api.CommitMessage) (ksuid.KSUID, error)
+	DeleteWhere(ctx context.Context, poolID ksuid.KSUID, branchName, src string, commit api.CommitMessage) (ksuid.KSUID, error)
 	Revert(ctx context.Context, poolID ksuid.KSUID, branch string, commitID ksuid.KSUID, commit api.CommitMessage) (ksuid.KSUID, error)
 	AddIndexRules(context.Context, []index.Rule) error
 	DeleteIndexRules(context.Context, []ksuid.KSUID) ([]index.Rule, error)
-	ApplyIndexRules(ctx context.Context, rule string, pool ksuid.KSUID, branchName string, ids []ksuid.KSUID) (ksuid.KSUID, error)
+	ApplyIndexRules(ctx context.Context, rules []string, pool ksuid.KSUID, branchName string, ids []ksuid.KSUID) (ksuid.KSUID, error)
 	UpdateIndex(ctx context.Context, names []string, pool ksuid.KSUID, branchName string) (ksuid.KSUID, error)
+	AddVectors(ctx context.Context, pool ksuid.KSUID, branch string, objects []ksuid.KSUID, message api.CommitMessage) (ksuid.KSUID, error)
+	DeleteVectors(ctx context.Context, pool ksuid.KSUID, branch string, objects []ksuid.KSUID, message api.CommitMessage) (ksuid.KSUID, error)
 }
 
 func OpenLake(ctx context.Context, u string) (Interface, error) {
@@ -58,6 +60,23 @@ func ScanIndexRules(ctx context.Context, api Interface) (zio.ReadCloser, error)
 	return api.Query(ctx, nil, "from :index_rules")
 }
 
+func GetIndexRules(ctx context.Context, api Interface) ([]index.Rule, error) {
+	r, err := ScanIndexRules(ctx, api)
+	if err != nil {
+		return nil, err
+	}
+	defer r.Close()
+	b := newBuffer(index.FieldRule{}, index.TypeRule{}, index.AggRule{})
+	if err := zio.Copy(b, r); err != nil {
+		return nil, err
+	}
+	var rules []index.Rule
+	for _, r := range b.results {
+		rules = append(rules, r.(index.Rule))
+	}
+	return rules, nil
+}
+
 func LookupPoolByName(ctx context.Context, api Interface, name string) (*pools.Config, error) {
 	b := newBuffer(pools.Config{})
 	zed := fmt.Sprintf("from :pools | name == '%s'", name)
@@ -83,6 +102,23 @@ func LookupPoolByName(ctx context.Context, api Interface, name string) (*pools.C
 	}
 }
 
+func GetPools(ctx context.Context, api Interface) ([]*pools.Config, error) {
+	b := newBuffer(pools.Config{})
+	q, err := api.Query(ctx, nil, "from :pools")
+	if err != nil {
+		return nil, err
+	}
+	defer q.Close()
+	if err := zio.Copy(b, zbuf.NoControl(q)); err != nil {
+		return nil, err
+	}
+	var pls []*pools.Config
+	for _, r := range b.results {
+		pls = append(pls, r.(*pools.Config))
+	}
+	return pls, nil
+}
+
 func LookupPoolByID(ctx context.Context, api Interface, id ksuid.KSUID) (*pools.Config, error) {
 	b := newBuffer(pools.Config{})
 	zed := fmt.Sprintf("from :pools | id == hex('%s')", idToHex(id))
