@@ -7,15 +7,16 @@ import (
 	"strings"
 
 	"github.com/brimdata/zed"
+	"github.com/brimdata/zed/compiler/ast"
 	"github.com/brimdata/zed/lake/branches"
 	"github.com/brimdata/zed/lake/commits"
 	"github.com/brimdata/zed/lake/data"
 	"github.com/brimdata/zed/lake/index"
 	"github.com/brimdata/zed/lake/journal"
-	"github.com/brimdata/zed/order"
+	"github.com/brimdata/zed/lakeparse"
 	"github.com/brimdata/zed/pkg/storage"
 	"github.com/brimdata/zed/runtime"
-	"github.com/brimdata/zed/zbuf"
+	"github.com/brimdata/zed/runtime/op"
 	"github.com/brimdata/zed/zio"
 	"github.com/brimdata/zed/zio/zngio"
 	"github.com/brimdata/zed/zson"
@@ -100,7 +101,7 @@ func loadMessage(objects []data.Object) string {
 
 func loadMeta(zctx *zed.Context, meta string) (*zed.Value, error) {
 	if meta == "" {
-		return &zed.Value{zed.TypeNull, nil}, nil
+		return zed.Null, nil
 	}
 	zv, err := zson.ParseValue(zed.NewContext(), meta)
 	if err != nil {
@@ -115,164 +116,114 @@ func (b *Branch) Delete(ctx context.Context, ids []ksuid.KSUID, author, message
 		if err != nil {
 			return nil, err
 		}
+		var objects []*data.Object
 		for _, id := range ids {
-			if !snap.Exists(id) {
-				return nil, fmt.Errorf("non-existent object %s: delete operation aborted", id)
+			o, err := snap.Lookup(id)
+			if err != nil {
+				return nil, err
 			}
+			objects = append(objects, o)
+		}
+		if message == "" {
+			var b strings.Builder
+			fmt.Fprintf(&b, "deleted %d data object%s\n\n", len(objects), plural(objects))
+			printObjects(&b, objects, maxMessageObjects)
+			message = b.String()
 		}
 		return commits.NewDeletesObject(parent.Commit, retries, author, message, ids), nil
 	})
 }
 
-func (b *Branch) DeleteByPredicate(ctx context.Context, c runtime.Compiler, src string, author, message, meta string) (ksuid.KSUID, error) {
+func (b *Branch) DeleteWhere(ctx context.Context, c runtime.Compiler, program ast.Op, author, message, meta string) (ksuid.KSUID, error) {
 	zctx := zed.NewContext()
 	appMeta, err := loadMeta(zctx, meta)
 	if err != nil {
 		return ksuid.Nil, err
 	}
-	val, op, err := c.ParseRangeExpr(zctx, src, b.pool.Layout)
-	if err != nil {
-		return ksuid.Nil, err
-	}
 	return b.commit(ctx, func(parent *branches.Config, retries int) (*commits.Object, error) {
+		pctx := op.NewContext(ctx, zctx, nil)
+		defer pctx.Cancel()
+		// XXX It would be great to not do this since and just pass the snapshot
+		// into c.NewLakeDeleteQuery since we have to load the snapshot later
+		// anyways. Unfortunately there's quite a few layers of plumbing needed
+		// to get this working in compiler.
+		commitish := &lakeparse.Commitish{
+			Pool:   b.pool.Name,
+			Branch: parent.Commit.String(),
+		}
+		query, err := c.NewLakeDeleteQuery(pctx, program, commitish)
+		if err != nil {
+			return nil, err
+		}
+		defer query.Pull(true)
+		w, err := NewWriter(ctx, zctx, b.pool)
+		if err != nil {
+			return nil, err
+		}
+		err = zio.CopyWithContext(ctx, w, query.AsReader())
+		if closeErr := w.Close(); err == nil {
+			err = closeErr
+		}
+		if err != nil {
+			return nil, err
+		}
 		base, err := b.pool.commits.Snapshot(ctx, parent.Commit)
 		if err != nil {
 			return nil, err
 		}
-		copies, deletes, err := b.getDbpObjects(ctx, c, val, op, parent.Commit)
-		if err != nil {
-			return nil, err
-		}
-		count := len(copies) + len(deletes)
-		if count == 0 {
-			return nil, errors.New("nothing to delete")
-		}
-		// XXX Copied should be deleted if the commit fails.
-		copied, err := b.dbpCopies(ctx, c, base, copies, src)
-		if err != nil {
-			return nil, err
+		deleted := query.DeletionSet()
+		if len(deleted) == 0 {
+			return nil, commits.ErrEmptyTransaction
 		}
 		patch := commits.NewPatch(base)
-		for _, o := range copied {
-			patch.AddDataObject(&o)
+		for _, oid := range deleted {
+			patch.DeleteObject(oid)
 		}
-		for _, id := range append(copies, deletes...) {
-			patch.DeleteObject(id)
+		for _, o := range w.Objects() {
+			obj := o
+			patch.AddDataObject(&obj)
 		}
 		if message == "" {
-			plural := "s"
-			if count == 1 {
-				plural = ""
+			var deletedObjs []*data.Object
+			for _, id := range deleted {
+				o, _ := base.Lookup(id)
+				deletedObjs = append(deletedObjs, o)
 			}
-			message = fmt.Sprintf("deleted %d object%s using predicate %s", count, plural, src)
+			var added []*data.Object
+			for _, o := range w.Objects() {
+				added = append(added, &o)
+			}
+			message = deleteWhereMessage(deletedObjs, added)
 		}
-		commit := patch.NewCommitObject(parent.Commit, retries, author, message, *appMeta)
-		return commit, nil
+		return patch.NewCommitObject(parent.Commit, retries, author, message, *appMeta), nil
 	})
 }
 
-func (b *Branch) dbpCopies(ctx context.Context, c runtime.Compiler, snap *commits.Snapshot, copies []ksuid.KSUID, filter string) ([]data.Object, error) {
-	if len(copies) == 0 {
-		return nil, nil
+func deleteWhereMessage(deleted, added []*data.Object) string {
+	var b strings.Builder
+	fmt.Fprintf(&b, "deleted %d data object%s\n\n", len(deleted), plural(deleted))
+	printObjects(&b, deleted, maxMessageObjects)
+	if len(added) > 0 {
+		fmt.Fprintf(&b, "\nadded %d data object%s\n\n", len(added), plural(added))
+		printObjects(&b, added, maxMessageObjects-len(deleted))
 	}
-	zctx := zed.NewContext()
-	readers := make([]zio.Reader, len(copies))
-	// XXX instead of opening each object individually like this we should
-	// instead create an ephemeral branch/pool that has only the copied
-	// objects then run our delete query on this subset.
-	for i, id := range copies {
-		o, err := snap.Lookup(id)
-		if err != nil {
-			return nil, err
-		}
-		// XXX Ideally we'd be opening objects via data.NewReader so we can
-		// leverage the seek index to skip over irrelavant segments but we don't
-		// have a way to translate the delete filter into an extent.Span, so
-		// just read the whole file.
-		reader, err := b.pool.engine.Get(ctx, o.SequenceURI(b.pool.DataPath))
-		if err != nil {
-			return nil, err
-		}
-		defer reader.Close()
-		readers[i] = zngio.NewReader(zctx, reader)
-		defer readers[i].(*zngio.Reader).Close()
-	}
-	// Keeps values that don't fit the filter by adding "not".
-	flowgraph, err := c.Parse("not " + filter)
-	if err != nil {
-		return nil, err
-	}
-	r := zio.NewCombiner(ctx, readers)
-	q, err := runtime.CompileQuery(ctx, zctx, c, flowgraph, []zio.Reader{r})
-	if err != nil {
-		return nil, err
-	}
-	defer q.Close()
-	w, err := NewWriter(ctx, zctx, b.pool)
-	if err != nil {
-		return nil, err
-	}
-	err = zbuf.CopyPuller(w, q)
-	if err2 := w.Close(); err == nil {
-		err = err2
-	}
-	if err != nil {
-		for _, o := range w.Objects() {
-			o.Remove(ctx, b.pool.engine, b.pool.DataPath)
-		}
-		return nil, err
-	}
-	return w.Objects(), nil
+	return b.String()
 }
 
-// getDbpObjects gets the object IDs of objects effected by a delete by predicate
-// operation.
-func (b *Branch) getDbpObjects(ctx context.Context, c runtime.Compiler, val *zed.Value, op string, commit ksuid.KSUID) ([]ksuid.KSUID, []ksuid.KSUID, error) {
-	const dbp = `
-const THRESH = %s
-from %s@%s:objects
-| {
-	id: id,
-	lower: compare(meta.first, meta.last, %t) < 0 ? meta.first : meta.last,
-	upper: compare(meta.first, meta.last, %t) < 0 ? meta.last : meta.first
-  }
-| switch (
-  case %s %s THRESH => deletes:=collect(id)
-  case %s %s THRESH => copies:=collect(id)
-)`
-	deletesField, copiesField := "upper", "lower"
-	if op == ">=" || op == ">" {
-		deletesField, copiesField = copiesField, deletesField
-	}
-	nullsMax := b.pool.Layout.Order == order.Asc
-	src := fmt.Sprintf(dbp, zson.MustFormatValue(val), b.pool.ID, commit, nullsMax, nullsMax, deletesField, op, copiesField, op)
-	flowgraph, err := c.Parse(src)
-	if err != nil {
-		return nil, nil, err
-	}
-	q, err := runtime.CompileLakeQuery(ctx, zed.NewContext(), c, flowgraph, nil, nil)
-	if err != nil {
-		return nil, nil, err
-	}
-	defer q.Close()
-	var objects struct {
-		Deletes []ksuid.KSUID `zed:"deletes"`
-		Copies  []ksuid.KSUID `zed:"copies"`
-	}
-	r := q.AsReader()
-	for {
-		val, err := r.Read()
-		if err != nil {
-			return nil, nil, err
-		}
-		if val == nil {
-			break
-		}
-		if err := zson.UnmarshalZNG(val, &objects); err != nil {
-			return nil, nil, err
+func printObjects(b *strings.Builder, objects []*data.Object, maxMessageObjects int) {
+	for k, o := range objects {
+		fmt.Fprintf(b, "  %s\n", o)
+		if k >= maxMessageObjects {
+			b.WriteString("  ...\n")
 		}
 	}
-	return objects.Copies, objects.Deletes, nil
+}
+
+func plural[S ~[]E, E any](s S) string {
+	if len(s) == 1 {
+		return ""
+	}
+	return "s"
 }
 
 func (b *Branch) Revert(ctx context.Context, commit ksuid.KSUID, author, message string) (ksuid.KSUID, error) {
@@ -321,6 +272,14 @@ func (b *Branch) CommitCompact(ctx context.Context, src, rollup []*data.Object,
 				return nil, err
 			}
 		}
+		if message == "" {
+			var b strings.Builder
+			fmt.Fprintf(&b, "compacted %d object%s\n\n", len(src), plural(src))
+			printObjects(&b, src, maxMessageObjects)
+			fmt.Fprintf(&b, "\ncreated %d object%s\n\n", len(rollup), plural(rollup))
+			printObjects(&b, rollup, maxMessageObjects-len(src))
+			message = b.String()
+		}
 		commit := patch.NewCommitObject(parent.Commit, retries, author, message, *appMeta)
 		return commit, nil
 	})
@@ -383,7 +342,7 @@ func (b *Branch) buildMergeObject(ctx context.Context, parent *branches.Config,
 	if err != nil {
 		return nil, fmt.Errorf("error merging %q into %q: %w", b.Name, parent.Name, err)
 	}
-	return diff.NewCommitObject(parent.Commit, retries, author, message, zed.Value{zed.TypeNull, nil}), nil
+	return diff.NewCommitObject(parent.Commit, retries, author, message, *zed.Null), nil
 }
 
 func commonAncestor(a, b []ksuid.KSUID) ksuid.KSUID {
@@ -556,3 +515,65 @@ func (b *Branch) indexObject(ctx context.Context, c runtime.Compiler, rules []in
 	}
 	return w.References(), err
 }
+
+func (b *Branch) AddVectors(ctx context.Context, ids []ksuid.KSUID, author, message string) (ksuid.KSUID, error) {
+	if message == "" {
+		message = vectorMessage("add", ids)
+	}
+	// XXX We should add some parallelism here to stream the next file while
+	// the CPU is chugging away on the current file.  See issue #4015.
+	for _, id := range ids {
+		if err := data.CreateVector(ctx, b.pool.engine, b.pool.DataPath, id); err != nil {
+			return ksuid.Nil, err
+		}
+	}
+	return b.commit(ctx, func(parent *branches.Config, retries int) (*commits.Object, error) {
+		snap, err := b.pool.commits.Snapshot(ctx, parent.Commit)
+		if err != nil {
+			return nil, err
+		}
+		for _, id := range ids {
+			if !snap.Exists(id) {
+				return nil, fmt.Errorf("non-existent object %s: vector add operation aborted", id)
+			}
+			if snap.HasVector(id) {
+				return nil, fmt.Errorf("vector exists for %s: vector add operation aborted", id)
+			}
+		}
+		return commits.NewAddVectorsObject(parent.Commit, author, message, ids, retries), nil
+	})
+}
+
+func (b *Branch) DeleteVectors(ctx context.Context, ids []ksuid.KSUID, author, message string) (ksuid.KSUID, error) {
+	if message == "" {
+		message = vectorMessage("delete", ids)
+	}
+	return b.commit(ctx, func(parent *branches.Config, retries int) (*commits.Object, error) {
+		snap, err := b.pool.commits.Snapshot(ctx, parent.Commit)
+		if err != nil {
+			return nil, err
+		}
+		for _, id := range ids {
+			if !snap.Exists(id) {
+				return nil, fmt.Errorf("non-existent object %s: vector delete operation aborted", id)
+			}
+			if !snap.HasVector(id) {
+				return nil, fmt.Errorf("vector %s does not exist: vector delete operation aborted", id)
+			}
+		}
+		return commits.NewDeleteVectorsObject(parent.Commit, author, message, ids, retries), nil
+	})
+}
+
+func vectorMessage(kind string, ids []ksuid.KSUID) string {
+	var b strings.Builder
+	b.WriteString("vector ")
+	b.WriteString(kind)
+	b.WriteString("\n\n")
+	for _, id := range ids {
+		b.WriteString("    ")
+		b.WriteString(id.String())
+		b.WriteByte('\n')
+	}
+	return b.String()
+}
