@@ -22,6 +22,7 @@ type Patch struct {
 	diff           *Snapshot
 	deletedObjects []ksuid.KSUID
 	deletedIndexes []indexRef
+	deletedVectors []ksuid.KSUID
 }
 
 var _ View = (*Patch)(nil)
@@ -55,6 +56,10 @@ func (p *Patch) LookupIndexObjectRules(id ksuid.KSUID) ([]index.Rule, error) {
 	return p.base.LookupIndexObjectRules(id)
 }
 
+func (p *Patch) HasVector(id ksuid.KSUID) bool {
+	return p.diff.HasVector(id) || p.base.HasVector(id)
+}
+
 func (p *Patch) Select(span extent.Span, o order.Which) DataObjects {
 	objects := p.base.Select(span, o)
 	objects.Append(p.diff.Select(span, o))
@@ -136,6 +141,26 @@ func (p *Patch) DeleteIndexObject(ruleID ksuid.KSUID, id ksuid.KSUID) error {
 	return nil
 }
 
+func (p *Patch) AddVector(id ksuid.KSUID) error {
+	if p.HasVector(id) {
+		return ErrExists
+	}
+	return p.diff.AddVector(id)
+}
+
+func (p *Patch) DeleteVector(id ksuid.KSUID) error {
+	if p.diff.HasVector(id) {
+		return p.diff.DeleteVector(id)
+	}
+	if !p.base.HasVector(id) {
+		return ErrNotFound
+	}
+	// Keep track of the deletions from the base so we can add the
+	// needed delete Actions when building the transaction patch.
+	p.deletedVectors = append(p.deletedVectors, id)
+	return nil
+}
+
 func (p *Patch) NewCommitObject(parent ksuid.KSUID, retries int, author, message string, meta zed.Value) *Object {
 	o := NewObject(parent, author, message, meta, retries)
 	for _, id := range p.deletedObjects {
@@ -150,11 +175,17 @@ func (p *Patch) NewCommitObject(parent ksuid.KSUID, retries int, author, message
 	for _, s := range p.diff.indexes.All() {
 		o.appendAddIndex(s)
 	}
+	for _, id := range p.deletedVectors {
+		o.appendDeleteVector(id)
+	}
+	for id := range p.diff.vectors {
+		o.appendAddVector(id)
+	}
 	return o
 }
 
 func (p *Patch) Revert(tip *Snapshot, commit, parent ksuid.KSUID, retries int, author, message string) (*Object, error) {
-	object := NewObject(parent, author, message, zed.Value{zed.TypeNull, nil}, retries)
+	object := NewObject(parent, author, message, *zed.Null, retries)
 	// For each data object that is added in the patch and is also in the tip, we do a delete.
 	for _, dataObject := range p.diff.SelectAll() {
 		if Exists(tip, dataObject.ID) {
