@@ -6,6 +6,12 @@ import (
 	"github.com/brimdata/zed/zio"
 )
 
+type Entry struct {
+	Key    *zed.Value `zed:"key"`
+	Count  uint64     `zed:"count"`
+	Offset int64      `zed:"offset"`
+}
+
 type Writer struct {
 	zctx    *zed.Context
 	builder *zcode.Builder
@@ -24,15 +30,15 @@ func NewWriter(w zio.Writer) *Writer {
 
 func (w *Writer) Write(key zed.Value, count uint64, offset int64) error {
 	b := w.builder
-	b.Reset()
+	b.Truncate()
 	b.Append(key.Bytes)
 	b.Append(zed.EncodeUint(count))
 	b.Append(zed.EncodeInt(offset))
 	if w.typ != key.Type {
 		var schema = []zed.Column{
-			{"key", key.Type},
-			{"count", zed.TypeUint64},
-			{"offset", zed.TypeInt64},
+			{Name: "key", Type: key.Type},
+			{Name: "count", Type: zed.TypeUint64},
+			{Name: "offset", Type: zed.TypeInt64},
 		}
 		w.recType = w.zctx.MustLookupTypeRecord(schema)
 		w.typ = key.Type
