@@ -1,9 +1,12 @@
 package agg
 
 import (
+	"fmt"
+
 	"github.com/axiomhq/hyperloglog"
 	"github.com/brimdata/zed"
 	"github.com/brimdata/zed/zcode"
+	"github.com/brimdata/zed/zson"
 )
 
 // DCount uses hyperloglog to approximate the count of unique values for
@@ -34,12 +37,21 @@ func (d *DCount) Result(*zed.Context) *zed.Value {
 	return zed.NewValue(zed.TypeUint64, zed.EncodeUint(d.sketch.Estimate()))
 }
 
-func (*DCount) ConsumeAsPartial(*zed.Value) {
-	// XXX this is straightforward to do using c.sketch.Merge().  See #1892.
-	panic("dcount: partials not yet implemented")
+func (d *DCount) ConsumeAsPartial(partial *zed.Value) {
+	if partial.Type != zed.TypeBytes {
+		panic(fmt.Errorf("dcount: partial has bad type: %s", zson.MustFormatValue(partial)))
+	}
+	var s hyperloglog.Sketch
+	if err := s.UnmarshalBinary(partial.Bytes); err != nil {
+		panic(fmt.Errorf("dcount: unmarshaling partial: %w", err))
+	}
+	d.sketch.Merge(&s)
 }
 
-func (*DCount) ResultAsPartial(zctx *zed.Context) *zed.Value {
-	// XXX this is straightforward to do using c.sketch.Merge().  See #1892.
-	panic("dcount: partials not yet implemented")
+func (d *DCount) ResultAsPartial(zctx *zed.Context) *zed.Value {
+	b, err := d.sketch.MarshalBinary()
+	if err != nil {
+		panic(fmt.Errorf("dcount: marshaling partial: %w", err))
+	}
+	return zed.NewBytes(b)
 }
