@@ -239,7 +239,7 @@ func newNumeric(lhs, rhs Evaluator) numeric {
 func enumify(val *zed.Value) *zed.Value {
 	// automatically convert an enum to its index value when coercing
 	if _, ok := val.Type.(*zed.TypeEnum); ok {
-		return &zed.Value{zed.TypeUint64, val.Bytes}
+		return zed.NewValue(zed.TypeUint64, val.Bytes)
 	}
 	return val
 }
@@ -423,7 +423,10 @@ func (a *Add) Eval(ectx Context, this *zed.Value) *zed.Value {
 	if zerr != nil {
 		return zerr
 	}
-	typ := zed.LookupPrimitiveByID(id)
+	typ, err := zed.LookupPrimitiveByID(id)
+	if err != nil {
+		return a.zctx.NewError(err)
+	}
 	switch {
 	case zed.IsFloat(id):
 		v1, v2 := a.operands.floats()
@@ -450,13 +453,20 @@ func (s *Subtract) Eval(ectx Context, this *zed.Value) *zed.Value {
 	if zerr != nil {
 		return zerr
 	}
-	typ := zed.LookupPrimitiveByID(id)
+	typ, err := zed.LookupPrimitiveByID(id)
+	if err != nil {
+		return s.zctx.NewError(err)
+	}
 	switch {
 	case zed.IsFloat(id):
 		v1, v2 := s.operands.floats()
 		return ectx.NewValue(typ, zed.EncodeFloat64(v1-v2))
 	case zed.IsSigned(id):
 		v1, v2 := s.operands.ints()
+		if id == zed.IDTime {
+			// Return the difference of two times as a duration.
+			typ = zed.TypeDuration
+		}
 		return ectx.NewValue(typ, zed.EncodeInt(v1-v2))
 	case zed.IsNumber(id):
 		v1, v2 := s.operands.uints()
@@ -473,7 +483,10 @@ func (m *Multiply) Eval(ectx Context, this *zed.Value) *zed.Value {
 	if zerr != nil {
 		return zerr
 	}
-	typ := zed.LookupPrimitiveByID(id)
+	typ, err := zed.LookupPrimitiveByID(id)
+	if err != nil {
+		return m.zctx.NewError(err)
+	}
 	switch {
 	case zed.IsFloat(id):
 		v1, v2 := m.operands.floats()
@@ -496,7 +509,10 @@ func (d *Divide) Eval(ectx Context, this *zed.Value) *zed.Value {
 	if zerr != nil {
 		return zerr
 	}
-	typ := zed.LookupPrimitiveByID(id)
+	typ, err := zed.LookupPrimitiveByID(id)
+	if err != nil {
+		return d.zctx.NewError(err)
+	}
 	switch {
 	case zed.IsFloat(id):
 		v1, v2 := d.operands.floats()
@@ -528,7 +544,10 @@ func (m *Modulo) Eval(ectx Context, this *zed.Value) *zed.Value {
 	if zerr != nil {
 		return zerr
 	}
-	typ := zed.LookupPrimitiveByID(id)
+	typ, err := zed.LookupPrimitiveByID(id)
+	if err != nil {
+		return m.zctx.NewError(err)
+	}
 	if zed.IsFloat(id) || !zed.IsNumber(id) {
 		return ectx.CopyValue(m.zctx.NewErrorf("type %s incompatible with '%%' operator", zson.FormatType(typ)))
 	}
@@ -753,7 +772,7 @@ func indexMap(zctx *zed.Context, ectx Context, typ *zed.TypeMap, mapBytes zcode.
 
 func deunion(ectx Context, typ zed.Type, b zcode.Bytes) *zed.Value {
 	if union, ok := typ.(*zed.TypeUnion); ok {
-		typ, b = union.SplitZNG(b)
+		typ, b = union.Untag(b)
 	}
 	return ectx.NewValue(typ, b)
 }
