@@ -8,6 +8,7 @@ import (
 	"github.com/brimdata/zed/pkg/field"
 	"github.com/brimdata/zed/pkg/nano"
 	"github.com/brimdata/zed/zcode"
+	"golang.org/x/exp/slices"
 )
 
 var (
@@ -152,12 +153,7 @@ func (v *Value) IsNull() bool {
 // Copy returns a copy of v that does not share v.Bytes.  The copy's Bytes field
 // is nil if and only if v.Bytes is nil.
 func (v *Value) Copy() *Value {
-	var b zcode.Bytes
-	if v.Bytes != nil {
-		b = make(zcode.Bytes, len(v.Bytes))
-		copy(b, v.Bytes)
-	}
-	return &Value{v.Type, b}
+	return &Value{v.Type, slices.Clone(v.Bytes)}
 }
 
 // CopyFrom copies from into v, reusing v.Bytes if possible and setting v.Bytes
@@ -167,8 +163,7 @@ func (v *Value) CopyFrom(from *Value) {
 	if from.Bytes == nil {
 		v.Bytes = nil
 	} else if v.Bytes == nil {
-		v.Bytes = make(zcode.Bytes, len(from.Bytes))
-		copy(v.Bytes, from.Bytes)
+		v.Bytes = slices.Clone(from.Bytes)
 	} else {
 		v.Bytes = append(v.Bytes[:0], from.Bytes...)
 	}
@@ -306,3 +301,22 @@ func (v *Value) MissingAsNull() *Value {
 	}
 	return v
 }
+
+func (v *Value) Under() *Value {
+	typ := v.Type
+	if _, ok := typ.(*TypeNamed); !ok {
+		if _, ok := typ.(*TypeUnion); !ok {
+			// common fast path
+			return v
+		}
+	}
+	bytes := v.Bytes
+	for {
+		typ = TypeUnder(typ)
+		union, ok := typ.(*TypeUnion)
+		if !ok {
+			return NewValue(typ, bytes)
+		}
+		typ, bytes = union.Untag(bytes)
+	}
+}
