@@ -150,7 +150,9 @@ func formatString(t *zed.TypeOfString, zv zcode.Bytes, inContainer bool) string
 	if bytes.Equal(zv, []byte{'-'}) {
 		return "\\x2d"
 	}
-
+	if string(zv) == "(empty)" {
+		return "\\x28empty)"
+	}
 	var out []byte
 	var start int
 	for i := 0; i < len(zv); {
@@ -193,8 +195,8 @@ func formatUnion(t *zed.TypeUnion, zv zcode.Bytes) string {
 	if zv == nil {
 		return FormatValue(zed.Null)
 	}
-	typ, iv := t.SplitZNG(zv)
-	s := strconv.FormatInt(int64(t.Selector(typ)), 10) + ":"
+	typ, iv := t.Untag(zv)
+	s := strconv.FormatInt(int64(t.TagOf(typ)), 10) + ":"
 	return s + formatAny(zed.NewValue(typ, iv), false)
 }
 
